// Star5 class file to be modified for StarField lab

// (c) Rebecca F. Bruce, J. Dean Brock 2003, 2004, 2007
// Department of Computer Science
// University of North Carolina at Asheville

package starfield;

import java.awt.*;
import java.awt.geom.* ;

/** Star5 class that must be modified during the StarField lab */
public class Star5 {
    
    // Parameters that can be set
    private int x ;
    private int y ;
    private int r ;     // radius
    private int a ;     // angle

    // default settings
    private static final int   DEFAULTX = 0 ;
    private static final int   DEFAULTY = 0 ;
    private static final int   DEFAULTR = 0 ;
    private static final int   DEFAULTA = 0 ;
    private static final Color DEFAULTC = Color.WHITE ;
    
    // Polygon for use in Java graphics class
    private Polygon points ;

    public Star5()
    {
        this(DEFAULTX, DEFAULTY, DEFAULTR, DEFAULTA) ;
    }

    public Star5(int x, int y) 
    {
        this(x, y, DEFAULTR, DEFAULTA) ;
    }
 
    public Star5(int x, int y, int radius) 
    {
        this(x, y, radius, DEFAULTA) ;
    }
 
    public Star5(int x, int y, int radius, int rotation) 
    {
        this.x = x ;
        this.y = y ;
        this.r = radius ;
        this.a = rotation ;
        computeStarPoints() ;
    }
    
    public int getX() {
        return x ;
    }
    public void setX(int x) {
        this.x = x ;
        computeStarPoints() ;
    }

    public int getY() {
        return y ;
    }
    public void setY(int y) {
        this.y = y ;
        computeStarPoints() ;
    }
    
    public int getRadius() {
        return r ;
    }
    public void setRadius(int radius) {
        r = radius ;
        computeStarPoints() ;
    }    
    
    public void draw(Graphics g)
    {
        Graphics newG = g.create() ;
        newG.setColor(DEFAULTC);
        newG.fillPolygon(points) ;
    }
    
    // WARNING -- nasty code ahead!
    //
    // You probably don't want to look at this....
    // Ratio of the radius of the inner Polygon of the star
    // to the radius of the whole star
    private static final double innerRatio = (3.0-Math.sqrt(5.0))/2.0 ;
    private void computeStarPoints() {
        double innerRad = innerRatio*r ;
        points = new Polygon() ;
        for (int i=0; i<5; ++i) {
            Point2D.Double outP = rotatedRootOfUnity(i,5) ;
            points.addPoint(x + (int)(r*outP.y), y - (int)(r*outP.x)) ;
            Point2D.Double inP =  rotatedRootOfUnity(2*i+1,10) ;
            points.addPoint(x + (int)(innerRad*inP.y), y - (int)(innerRad*inP.x)) ;
        }
    }
    
    private Point2D.Double rotatedRootOfUnity(int k, int n)
    // Returns k'th n'th root of unity with a "twist" at a degrees
    // rootOfUnity(0,n) is (1,0) for the graphics display....
    {
	double angle = ((k * 2.0 / n) + (a/180.0)) * Math.PI ;
	return new Point2D.Double(Math.cos(angle), Math.sin(angle)) ;
    }
}
